import struct

# binary format is according to Perforce://Measurements/FlexRIO/HWCode/BlackadderFams/Chimera/MicroblazeFW/master/doc/ChimeraPoscFlashMap.xlsx

SIZE_BYTES = 4
CRC_BYTES = 4
CMD_HEADER_BYTES = 2
READ_SIZE_BYTES = 1
SLEEP_TIME_BYTES = 2

WRITE_CMD_BYTES = SIZE_BYTES + CMD_HEADER_BYTES + CRC_BYTES
READ_CMD_BYTES = SIZE_BYTES + CMD_HEADER_BYTES + READ_SIZE_BYTES + CRC_BYTES
SLEEP_CMD_BYTES = SIZE_BYTES + CMD_HEADER_BYTES + SLEEP_TIME_BYTES + CRC_BYTES
EOS_CMD_BYTES = SIZE_BYTES + CMD_HEADER_BYTES + CRC_BYTES

MAX_CMD_SIZE = 4096


class PoscBinary(object):
    def __init__(self, binary_file):
        self.binary_file = binary_file

    def write_i2c(self, bus_instance, slave_addr, write_data, repeated_start=False):
        size = struct.pack('<i', len(write_data) + WRITE_CMD_BYTES)  # 32-bit size is byte length of data plus command overhead
        if (len(write_data) + WRITE_CMD_BYTES > MAX_CMD_SIZE):
            raise ValueError('Max command size of 4096 exceeded')
        crc = size  # CRC Placeholder; CRC calculation not supported in initial release; Feature: AzDo 1201555
        write_command = b'\x02'
        command_bytes = size + write_command + struct.pack('<B', slave_addr) + write_data + crc  # concatenate bytes
        self.binary_file.write(command_bytes)

    def transfer_i2c(self, bus_instance, slave_addr, write_data, num_bytes_to_read, repeated_start=False):
        size = struct.pack('<i', len(write_data) + READ_CMD_BYTES)  # 32-bit size is byte length of write data plus command overhead
        if (len(write_data) + READ_CMD_BYTES > MAX_CMD_SIZE):
            raise ValueError('Max command size of 4096 exceeded')
        crc = size  # CRC Placeholder; CRC calculation not supported in initial release; Feature: AzDo 1201555
        read_command = b'\x01'
        command_bytes = size + read_command + struct.pack('<B', slave_addr) + struct.pack('<B', num_bytes_to_read) + write_data + crc  # concatenate bytes
        self.binary_file.write(command_bytes)
        return [num_bytes_to_read]

    def sleep(self, seconds):
        size = struct.pack('<i', SLEEP_CMD_BYTES)  # 32-bit size is constant for sleep command
        crc = size  # CRC Placeholder; CRC calculation not supported in initial release; Feature: AzDo 1201555
        wait_command = struct.pack('<H', 0)  # wait command includes 'Don't Care' byte for the slave address
        command_bytes = size + wait_command + struct.pack('<H', int(seconds * 1000)) + crc  # concatenate bytes
        self.binary_file.write(command_bytes)

    def end_of_script(self):
        # Add end of script command section
        size = struct.pack('<i', EOS_CMD_BYTES)  # 32-bit size is constant for "end of script" command
        crc = size  # CRC Placeholder; CRC calculation not supported in initial release; Feature: AzDo 1201555
        end_command = b'\x03'
        command_bytes = size + end_command + b'\x00' + crc  # concatenate bytes
        self.binary_file.write(command_bytes)
        self.binary_file.close
